import sys
import libvirt
import time
import config
import threading
import socket

conn = libvirt.open('qemu:///system')
if conn == None:
    print('Failed to open connection to qemu:///system', file=sys.stderr)
    exit(1)

domainName = 'server1'
dom = conn.lookupByName(domainName)
if dom == None:
    print('Failed to get the domain object', file=sys.stderr)

f1 = open('server_ips.txt', 'w')
f1.write(config.new_server_ips[0])
f1.close()

def establish_conn(server_ip, port):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(10)
    while(True):
        try:
            sock.connect((server_ip, port))
            break
        # except socket.gaierror, e:
        #     print "Address-related error connecting to server: %s" % e
        #     sys.exit(1)
        except socket.error:
            print("Connection error: {}".format(socket.error))
    print("Connection established...")
    sock.settimeout(None)
    sock.close()

print("Monitor program started running...")
ind = 0
while(True):
    N = 10
    cpu_time1 = 0
    cpu_time2 = 0
    cpu_stats1 = dom.getCPUStats(True)
    for (i, cpu) in enumerate(cpu_stats1):
        cpu_time1 = cpu['cpu_time'] / 1000000000.
        # print('CPU '+str(i)+' Time: '+str(cpu['cpu_time'] / 1000000000.))
    time.sleep(N)
    cpu_stats2 = dom.getCPUStats(True)
    for (i, cpu) in enumerate(cpu_stats2):
        cpu_time2 = cpu['cpu_time'] / 1000000000.
        # print('CPU '+str(i)+' Time: '+str(cpu['cpu_time'] / 1000000000.))
    cpu_usage = 100 * (cpu_time2 - cpu_time1) / N
    print(cpu_usage)
    if(cpu_usage > 60.0):
        print('Overload detected')
        dom1 = conn.lookupByName('server2')
        flag = dom1.isActive()
        f = open('server2.xml', 'r')
        xmlconfig = " ".join(f.readlines())
        ind += 1
        # print(xmlconfig)
        # t1 = None
        if(flag == False): 
            dom1 = conn.defineXML(xmlconfig)
            if dom1 == None:
                print('Failed to define a domain from an XML definition.', file=sys.stderr)
                exit(1)
            if dom1.create() < 0:
                print('Can not boot guest domain.', file = sys.stderr)
                exit(1)
            print('Guest ' + dom1.name() + ' has booted', file = sys.stderr)
            # while(dom1.isActive() == False):
            #     continue
            for lease in conn.networkLookupByName("default").DHCPLeases():
                if lease['ipaddr'] not in config.new_server_ips:
                    config.new_server_ips.append(lease['ipaddr'])
                    t1 = threading.Thread(target = establish_conn, args = (lease['ipaddr'], 9001))
                    t1.start()
        else :
            if((dom1.isActive() == True) and (ind >= 3)):
                t1.join()
                f1 = open('server_ips.txt', 'w')
                for i in range(len(config.new_server_ips)):
                    f1.write(config.new_server_ips[i] + '\n')
                f1.close()
                print(config.new_server_ips)
            
conn.close()
exit(0)